﻿using System;
using System.Diagnostics;
using System.IO;
using System.Windows;
using Telerik.Windows.Documents.Fixed;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.UI.Extensibility;

namespace HighlightsUILayerDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private const string ResourceFilePath = "/HighlightsUILayerDemo;component/Resources/TestFileWithHighlights.pdf";
        private const string SaveFileName = "TestFileWithHighlights.pdf";
        private readonly PdfFormatProvider pdfFormatProvider;
        private readonly HighlightsManager highlightsManager;

        public MainWindow()
        {
            this.highlightsManager = new HighlightsManager();
            ExtensibilityManager.RegisterLayersBuilder(new CustomLayersBuilder(this.highlightsManager));

            InitializeComponent();
            this.pdfFormatProvider = new PdfFormatProvider();
            this.pdfViewer.Mode = Telerik.Windows.Documents.Fixed.UI.FixedDocumentViewerMode.TextSelection;
            this.pdfViewer.DocumentChanged += this.PdfViewerDocumentChanged;
        }

        private void SaveAndView_Click(object sender, RoutedEventArgs e)
        {
            RadFixedDocument document = this.ImportDocument();

            for (int pageIndex = 0; pageIndex < document.Pages.Count; pageIndex++)
            {
                var path = new Telerik.Windows.Documents.Fixed.Model.Graphics.Path();
                path.IsFilled = true;
                path.IsStroked = false;
                path.Fill = new RgbColor(255, 255, 0);
                path.Geometry = this.ConvertGeometry(this.highlightsManager.GetHighlights(pageIndex));
                document.Pages[pageIndex].Content.Insert(0, path);
            }

            this.SaveAndView(document);
        }

        private void AddHighlights_Click(object sender, RoutedEventArgs e)
        {
            RadFixedDocument document = this.pdfViewer.Document;
            int startPageIndex =  document.Pages.IndexOf(document.Selection.StartPosition.Page);
            int endPageIndex = document.Pages.IndexOf(document.Selection.EndPosition.Page);

            for (int pageIndex = startPageIndex; pageIndex <= endPageIndex; pageIndex++)
            {
                System.Windows.Media.PathGeometry geometry = document.Selection.GetSelectionGeometry(this.pdfViewer.Document.Pages[pageIndex]);

                foreach (System.Windows.Media.PathFigure figure in geometry.Figures)
                {
                    this.highlightsManager.AddHighlight(pageIndex, figure);
                }
            }
        }

        private void SaveAndView(RadFixedDocument document)
        {
            if (File.Exists(SaveFileName))
            {
                File.Delete(SaveFileName);
            }

            using (Stream fileStream = new FileStream(SaveFileName, FileMode.OpenOrCreate))
            {
                this.pdfFormatProvider.Export(document, fileStream);
            }

            Process.Start(SaveFileName);
        }

        private RadFixedDocument ImportDocument()
        {
            using (Stream fileStream = Application.GetResourceStream(new Uri(ResourceFilePath, UriKind.Relative)).Stream)
            {
                return this.pdfFormatProvider.Import(fileStream);
            }
        }

        private PathGeometry ConvertGeometry(System.Windows.Media.PathGeometry geometry)
        {
            PathGeometry pathGeometry = new PathGeometry();

            foreach (var figure in geometry.Figures)
            {
                PathFigure pathFigure = pathGeometry.Figures.AddPathFigure();
                pathFigure.StartPoint = figure.StartPoint;
                pathFigure.IsClosed = true;

                var polyline = (System.Windows.Media.PolyLineSegment)figure.Segments[0];

                foreach (Point point in polyline.Points)
                {
                    pathFigure.Segments.AddLineSegment(point);
                }
            }

            return pathGeometry;
        }

        private void PdfViewerDocumentChanged(object sender, DocumentChangedEventArgs e)
        {
            this.highlightsManager.Clear();
        }
    }
}
